<?php
/*
 * tweeper - a Twitter to RSS web scraper
 * 
 * Copyright (C) 2013-2014  Antonio Ospite <ao2@ao2.it>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

date_default_timezone_set('UTC');

class Tweeper {

  private static $USER_AGENT = "Mozilla/5.0 (Windows NT 6.1; rv:22.0) Gecko/20130405 Firefox/22.0";

  public function __construct($stylesheet, $generate_enclosure = FALSE) {
    $stylesheet_contents = $this->get_contents($stylesheet);

    $xslDoc = new DOMDocument();
    $xslDoc->loadXML($stylesheet_contents);

    $this->xsltProcessor = new XSLTProcessor();
    $this->xsltProcessor->registerPHPFunctions();
    $this->xsltProcessor->setParameter('', 'generateEnclosure', $generate_enclosure);
    $this->xsltProcessor->importStylesheet($xslDoc);
  }

  public static function epoch_to_gmdate($timestamp)
  {
    return gmdate('D, d M Y H:i:s', $timestamp) . ' GMT';
  }

  public static function str_to_gmdate($date)
  {
    $timestamp = strtotime($date);
    return Tweeper::epoch_to_gmdate($timestamp);
  }

  private static function get_contents($url)
  {
    $ch = curl_init($url);
    curl_setopt_array($ch, array(
      CURLOPT_HEADER => FALSE,
      CURLOPT_FOLLOWLOCATION => TRUE,  // follow http redirects to get the real URL
      CURLOPT_RETURNTRANSFER => TRUE,
      CURLOPT_SSL_VERIFYHOST => FALSE,
      CURLOPT_SSL_VERIFYPEER => FALSE,
      CURLOPT_HTTPHEADER => array('Accept-language: en'),
      CURLOPT_USERAGENT => Tweeper::$USER_AGENT,
    ));
    $contents = curl_exec($ch);
    curl_close($ch);

    return $contents;
  }

  private static function get_info($url)
  {
    $ch = curl_init($url);
    curl_setopt_array($ch, array(
      CURLOPT_HEADER => TRUE,
      CURLOPT_NOBODY => TRUE,
      CURLOPT_FOLLOWLOCATION => TRUE,  // follow http redirects to get the real URL
      CURLOPT_RETURNTRANSFER => TRUE,
      CURLOPT_SSL_VERIFYHOST => FALSE,
      CURLOPT_SSL_VERIFYPEER => FALSE,
      CURLOPT_USERAGENT => Tweeper::$USER_AGENT,
    ));
    curl_exec($ch);
    $url_info = curl_getinfo($ch);
    curl_close($ch);

    return $url_info;
  }

  public static function generate_enclosure($url)
  {
    $supported_content_types = array(
      "application/ogg",
      "audio/aac",
      "audio/mp4",
      "audio/mpeg",
      "audio/ogg",
      "audio/vorbis",
      "audio/wav",
      "audio/webm",
      "audio/x-midi",
      "image/gif",
      "image/jpeg",
      "video/avi",
      "video/mp4",
      "video/mpeg",
      "video/ogg",
    );

    $url_info = Tweeper::get_info($url);

    $supported = in_array($url_info['content_type'], $supported_content_types);
    if (!$supported) {
      error_log("Unsupported enclosure content type \"" . $url_info['content_type'] . "\" for URL: " . $url_info['url']);
      return '';
    }

    $dom = new DomDocument();
    $enc = $dom->createElement('enclosure');
    $enc->setAttribute('url', $url_info['url']);
    $enc->setAttribute('length', $url_info['download_content_length']);
    $enc->setAttribute('type', $url_info['content_type']);

    $dom->appendChild($enc);

    return $dom->saveXML($enc);
  }

  /* Mimic the message from libxml.c::php_libxml_ctx_error_level() */
  private function log_xml_error($error) {
    $output = "";

    switch ($error->level) {
    case LIBXML_ERR_WARNING:
      $output .= "Warning $error->code: ";
      break;
    case LIBXML_ERR_ERROR:
      $output .= "Error $error->code: ";
      break;
    case LIBXML_ERR_FATAL:
      $output .= "Fatal Error $error->code: ";
      break;
    }

    $output .= trim($error->message);

    if ($error->file) {
      $output .= " in $error->file";
    } else {
      $output .= " in Entity,";
    }

    $output .=" line $error->line";

    error_log($output);
  }

  public function tweep($uri) {
    $html = Tweeper::get_contents($uri);

    $xmlDoc = new DOMDocument();

    // Handle warnings and errors when loading invalid HTML.
    $xml_errors_value = libxml_use_internal_errors(true);
    $xmlDoc->loadHTML($html);
    foreach (libxml_get_errors() as $xml_error) {
      $this->log_xml_error($xml_error);
    }
    libxml_clear_errors();
    libxml_use_internal_errors($xml_errors_value);

    $output = $this->xsltProcessor->transformToXML($xmlDoc);

    if (FALSE === $output) {
      trigger_error('XSL transformation failed.', E_USER_ERROR);
      return NULL;
    }
    return $output;
  }
}

function usage($argv)
{
  if (php_sapi_name() != 'cli')
    $usage = htmlentities("{$_SERVER['SCRIPT_NAME']}?src_url=<src_url>&generate_enclosure=<0|1>");
  else
    $usage = "{$argv[0]} [-e|-h|--help] <src_url>\n";

  return "usage: $usage";
}

function parse_options_cli($argv, $argc)
{
  $options = array(
    'generate_enclosure' => FALSE
  );

  if ($argc < 2)
    return $options;

  $cli_options = getopt("eh", array("help"));
  foreach ($cli_options as $opt => $val) {
    switch ($opt) {
    case 'e':
      $options['generate_enclosure'] = TRUE;
      break;
    case 'h':
    case 'help':
      echo usage($argv);
      exit(0);
    default:
      fwrite(STDERR, usage($argv));
      exit(1);
    }
  }

  $options['src_url'] = $argv[count($cli_options) + 1];

  return $options;
}

function parse_options_query_string()
{
  $options = array(
    'generate_enclosure' => FALSE
  );

  if (isset($_GET['src_url']))
    $options['src_url'] = $_GET['src_url'];

  if (isset($_GET['generate_enclosure']))
    $options['generate_enclosure'] = $_GET['generate_enclosure'] == 1;

  return $options;
}


if (php_sapi_name() != 'cli') {
  $options = parse_options_query_string();
  $ERROR_STREAM = fopen('php://output', 'w');
} else {
  $options = parse_options_cli($argv, $argc);
  $ERROR_STREAM = fopen('php://stderr', 'w');
}

if (!isset($options['src_url'])) {
  fwrite($ERROR_STREAM, usage($argv));
  exit(1);
}

$url = parse_url($options['src_url']);
if (FALSE === $url || empty($url["host"])) {
  fwrite($ERROR_STREAM, "Invalid url: ${options['src_url']}\n");
  exit(1);
}

$stylesheet = "file://" . __DIR__ . "/rss_converter_" . $url["host"] . ".xsl";
if (FALSE === file_exists($stylesheet)) {
  fwrite($ERROR_STREAM, "Conversion to RSS not supported: {$url["host"]}\n");
  exit(1);
}

$tweeper = new Tweeper($stylesheet, $options['generate_enclosure']);
echo $tweeper->tweep($options['src_url']);
